'''
avlib 0.4

(c) 2003 Jan ONDREJ (SAL) <ondrejj@salstar.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
    
'''

import os,sys,re,string,time,random,traceback,signal

BUFSIZE=10240
ROOT_PATH=''
MASTER_PID=os.getpid()
INT_DEBUG_LEVEL=2

class ScannerError(Exception):
      """Any Error Of Any Scanner"""

def set_logfile(logfile):
    global logfd
    if logfile!='-':
      logfd=open(logfile,'a',0)
      os.chmod(logfile,0640)
    else:
      logfd=os.fdopen(os.dup(1),'w',0)

def set_debug_level(level):
    global INT_DEBUG_LEVEL
    INT_DEBUG_LEVEL=level

def debug(level,*s):
    if INT_DEBUG_LEVEL>=level:
      o=''
      for i in s:
        if type(i)==type([]):
          for x in i:
            o=o+str(x)
        else:
          o=o+str(i)
      p=os.getpid()
      if p==MASTER_PID:
        logfd.write(o+'\n')
      else:
        logfd.write("%6s: %s\n" % (p,o))
    
def debug_traceback(level,arg):
    debug(level,arg+str(traceback.format_exception(sys.exc_type,
                      sys.exc_value, sys.exc_traceback)))

def traceback_value_str():
    return string.join(traceback.format_exception_only(
                           sys.exc_type, sys.exc_value),', ')

class popen:
  def __init__(self, cmd, wd=''):
      self.exitstatus,self.killsignal,self.execerror,self.pid=-1,-1,'',0
      self.pcr,self.pcw = os.pipe()
      self.cpr,self.cpw = os.pipe()
      self.oldsigchld=signal.getsignal(signal.SIGCHLD)
      signal.signal(signal.SIGUSR1,self.sighandler)
      signal.signal(signal.SIGCHLD,self.sighandler)
      self.pid = os.fork()
      if self.pid == 0: # child
        signal.signal(signal.SIGUSR1,signal.SIG_IGN)
        signal.signal(signal.SIGCHLD,signal.SIG_IGN)
        os.close(self.pcw)
        os.close(self.cpr)
        os.dup2(self.pcr,0)
        os.dup2(self.cpw,1)
        os.dup2(self.cpw,2)
        try:
          if wd:
            os.chdir(wd)
          debug(4,"POpen: Running: ",str(cmd))
          os.execvp(cmd[0],cmd)
        except os.error,(ec,es):
          debug(3,"POpen: os.error: ",es+str([cmd[0]]))
          os.write(1,es+str([cmd[0]]))
        os.kill(os.getppid(),signal.SIGUSR1)
        os._exit(10)
      os.close(0);os.close(1);os.close(2)
      os.close(self.pcr)
      os.close(self.cpw)
      self.tocmd=os.fdopen(self.pcw, 'w', 0)
      self.fromcmd=os.fdopen(self.cpr, 'r', 0)
  def sighandler(self,sn,stack):
      if sn==signal.SIGUSR1:
        self.execerror=os.read(self.cpr,1024)
      else:
        self.wait()
        debug(4,"POpen SIG=",sn,", pid=",self.pid,", exitstatus=",self.exitstatus,
              ", signal=",self.killsignal)
  def wait(self):
      try:
        pid,es=os.waitpid(self.pid,0)
        self.exitstatus=es/256
        self.killsignal=es%256
      except OSError:
        if self.exitstatus<0:
          self.exitstatus=0 # pid already exited, but no exit status
      signal.signal(signal.SIGCHLD,self.oldsigchld)
      return self.exitstatus
  def readlines(self):
      if self.exitstatus<0:
        try:
          return self.fromcmd.readlines()
        except IOError,(ec,es):
          if ec==4: #Interrupted system call
            return []
          else:
            raise
      else:
        return []
  def write(self,s):
      if self.exitstatus<0:
        return self.tocmd.write(s)
      else:
        return 0
  def close(self,rw=3):
      if rw&1:
        self.tocmd.flush()
        os.close(self.pcw)
      if rw&2:
        os.close(self.cpr)

def rlistdir(rootpath,path=''):
    files=[]
    try:
      listdir=os.listdir(os.path.join(rootpath,path))
    except:
      return []
    for d in os.listdir(os.path.join(rootpath,path)):
      if os.lstat(os.path.join(rootpath,path,d))[0] & 0x4000: # is_dir
        files=files+rlistdir(rootpath,os.path.join(path,d))
      else:
        files.append(os.path.join(path,d))
    return files

def fromhdr():
    return "From local@local.local"+\
      time.strftime("  %c\r\n",time.localtime(time.time()))

class mktemp:
      chars='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
      def __init__(self,prefix,suffix='',flags='w',umode=-1):
          self.flag_autorm=0
          self.flags=flags
          while 1:
            s=prefix
            r=random.Random()
            for i in range(10):
              s=s+self.chars[int(r.random()*len(self.chars))]
            self.name=ROOT_PATH+s+suffix
            try:
              if flags=='d':
                if umode<0:
                  umode=0700
                os.mkdir(self.name)
                os.chmod(self.name,umode)
                if (os.stat(self.name)[0] & 0170000)==040000:
                  return
              else:
                if umode<0:
                  umode=0600
                self.fd=os.open(self.name,os.O_WRONLY|os.O_CREAT|os.O_EXCL,umode)
                self.f=os.fdopen(self.fd,'w',BUFSIZE)
                if (os.stat(self.name)[0] & 0170000)==0100000:
                  return
            except OSError,(ec,es):
              if not (ec in [17,21]):
                raise
      def __del__(self):
          if self.flag_autorm:
            self.rm
      def rm(self):
          if self.flags=='d':
            os.rmdir(self.name)
          else:
            try:
              f.close()
            except:
              pass
            os.unlink(self.name)
      def autorm(self):
          self.flag_autorm=1
          return self

def creatembox(f,data,addheader=0):
    try:
      if addheader:
        f.write("From local@local"+\
          time.strftime("  %c\n",time.localtime(time.time())))
      f.write(data)
      f.close()
    except os.error,(ec,es):
      debug(0,'creatembox: ERROR: Open: ',es,[[file]])
      raise
    except IOError,(ec,es):
      debug(0,'creatembox: ERROR: IO: ',es,[[file]])
      raise

def tempcleanup(file,s=''):
    try:
      os.unlink(file)
    except OSError,(er,es):
      debug(0,s+": ERROR: OSError: ",es,file)

def scan_over(scanner,streams):
    detected,virlist='',[]
    for stream in streams:
      d,v=clamscanf(stream,self.arg)
      if not detected:
        detected=d
      virlist=virlist+v
    return detected,virlist

class ascanner:
  name='AScanner'
  is_spamscan=0
  is_multistream=0
  def __init__(self,arg):
      self.arg=arg
  def destroy(self):
      pass
  def scanstream(self,stream):
      raise ScannerError,'Not implemented'
  def scanfile(self,files,dirname=''):
      raise ScannerError,'Not implemented'
  def reinit(self):
      pass

class interscanner(ascanner):
  name='InterScanner'
  scanners=[]
  def __init__(self,scanner):
      self.scanner=scanner
  def destroy(self):
      self.scanner.destroy()
  def reinit(self):
      self.scanner.reinit()
